<?php

/*
Description:
This is a package manager & automod tool for OpenCart 1.x
It should work with versions 1.4.0 and later.
Use the upload field to upload a zip file that follows the correct package format.
If you have modifications that need to be patched into other files you can use the xml or patch method
If using the xml method, be sure to include an install.xml file with the correct xml formatting.
ex:
-----------------------------------------------
<modification>
	<id>Latest Products SideBox</id>
	<version>1.0.0</version>
	<author>qphoria</author>
	<file name="catalog/model/catalog/product.php">
		<operation>
			<search position="before|after|replace">

			</search>
			<add>

			</add>
		</operation>
	</file>
	<file name="catalog/view/theme/default/template/product/product.tpl">
		<operation>
			<search position="before|after|replace">

			</search>
			<add>

			</add>
		</operation>
	</file>
</modification>
------------------------------------------------

Author:
Qphoria@gmail.com

Web:
http://www.theQdomain.com
*/


class ControllerToolPackageManager extends Controller {
	private $error = array();

	public function index() {

		if (!isset($this->session->data['token'])) {
			$this->session->data['token'] = 0;
		}
		$this->data['token'] = $this->session->data['token'];

		$this->load->language('tool/package_manager');

		$this->document->title = $this->language->get('heading_title');

		$this->load->model('tool/package_manager');

		if ($this->request->server['REQUEST_METHOD'] == 'POST' && $this->validate()) {

			// Upload file and create the extract path based on the filename
			if (isset($this->request->files['upload']['tmp_name'])) {
				$basepath = dirname(DIR_SYSTEM);
				$file = pathinfo($this->request->files['upload']['name']);
				$extractpath = $basepath . '/zip/' . str_replace(' ', '', ucwords(preg_replace('/[^(\x20-\x7F)\x0A]*/','', $file['filename'])));
				if ($this->zip_extract($this->request->files['upload']['tmp_name'], $extractpath)) {

					// if the file has an install.xml file, use that for automatic file modifications
					if (file_exists($extractpath . '/install.xml')) {
						$dom = new DOMDocument('1.0', 'UTF-8');
						$result = $dom->load($extractpath . '/install.xml');
						$modification = $dom->getElementsByTagName('modification')->item(0);
						$id = $modification->getElementsByTagName('id')->item(0)->nodeValue;
						$version = $modification->getElementsByTagName('version')->item(0)->nodeValue;
						$files = $modification->getElementsByTagName('file');

						foreach ($files as $file) {
							$filename = $basepath.'/'.$file->getAttribute('name');
							if (!file_exists($filename)) { continue; }
							$error_type = $file->getAttribute('error');
							$operations = $modification->getElementsByTagName('operation');
							foreach ($operations as $operation) {
								$search = $modification->getElementsByTagName('search')->item(0)->nodeValue;
								$position = $file->getAttribute('position');
								$add = $modification->getElementsByTagName('add')->item(0)->nodeValue;

								// Edit the file and perform changes
								$lines = file($filename);


							}

						}
					}

					// Copy files from the extract path to the store path. Backup files if exist
					echo $this->recurse_copy ($extractpath, $basepath);

					//$this->model_tool_package_manager->restore($content);

					$this->session->data['success'] = $this->language->get('text_success');

					$this->redirect(HTTPS_SERVER . 'index.php?route=tool/package_manager&token=' . $this->session->data['token']);
				} else {
					$this->error['warning'] = $this->language->get('error_empty');
				}
			}
		}

		$this->data['heading_title'] = $this->language->get('heading_title');

		$this->data['text_select_all'] = $this->language->get('text_select_all');
		$this->data['text_unselect_all'] = $this->language->get('text_unselect_all');

		$this->data['entry_restore'] = $this->language->get('entry_restore');
		$this->data['entry_upload'] = $this->language->get('entry_upload');

		$this->data['button_upload'] = $this->language->get('button_upload');

		$this->data['tab_general'] = $this->language->get('tab_general');

 		if (isset($this->error['warning'])) {
			$this->data['error_warning'] = $this->error['warning'];
		} else {
			$this->data['error_warning'] = '';
		}

		if (isset($this->session->data['success'])) {
			$this->data['success'] = $this->session->data['success'];

			unset($this->session->data['success']);
		} else {
			$this->data['success'] = '';
		}

  		$this->document->breadcrumbs = array();

   		$this->document->breadcrumbs[] = array(
       		'href'      => HTTPS_SERVER . 'index.php?route=common/home&token=' . $this->session->data['token'],
       		'text'      => $this->language->get('text_home'),
      		'separator' => FALSE
   		);

   		$this->document->breadcrumbs[] = array(
       		'href'      => HTTPS_SERVER . 'index.php?route=tool/package_manager&token=' . $this->session->data['token'],
       		'text'      => $this->language->get('heading_title'),
      		'separator' => ' :: '
   		);

		$this->data['upload'] = HTTPS_SERVER . 'index.php?route=tool/package_manager&token=' . $this->session->data['token'];

		$this->load->model('tool/package_manager');

		$this->data['tables'] = $this->model_tool_package_manager->getTables();

		$this->template = 'tool/package_manager.tpl';
		$this->children = array(
			'common/header',
			'common/footer'
		);

		$this->response->setOutput($this->render(TRUE), $this->config->get('config_compression'));
	}

	public function upload() {
		if ($this->request->server['REQUEST_METHOD'] == 'POST' && $this->validate()) {
			$this->response->addheader('Pragma: public');
			$this->response->addheader('Expires: 0');
			$this->response->addheader('Content-Description: File Transfer');
			$this->response->addheader('Content-Type: application/octet-stream');
			$this->response->addheader('Content-Disposition: attachment; filename=package_manager.sql');
			$this->response->addheader('Content-Transfer-Encoding: binary');

			$this->load->model('tool/package_manager');

			$this->response->setOutput($this->model_tool_package_manager->package_manager($this->request->post['package_manager']));
		} else {
			return $this->forward('error/error_404', 'index');
		}
	}

	private function validate() {
		if (!$this->user->hasPermission('modify', 'tool/package_manager')) {
			$this->error['warning'] = $this->language->get('error_permission');
		}

		if (!$this->error) {
			return TRUE;
		} else {
			return FALSE;
		}
	}

	private function zip_extract($file, $extractPath) {

	    $zip = new ZipArchive;
	    if ($zip->open($file) === TRUE) {
	        $zip->extractTo($extractPath);
	        $zip->close();
	        return TRUE;
	    } else {
	        return FALSE;
	    }

	}

	function recurse_copy($src, $dst, $backup = true) {
	    $dir = opendir($src);
	    if (!file_exists($dst)) { mkdir($dst, 0777, true); }
	    while(false !== ($file = readdir($dir))) {
	        if (($file != '.') && ($file != '..')) {
	            if (is_dir($src . '/' . $file)) {
	                $this->recurse_copy($src . '/' . $file, $dst . '/' . $file);
	            }
	            else {

					// Check if file exists and make a copy for backup with time/date stamp
					if (is_file($dst . '/' . $file) && file_exists($dst . '/' . $file)) {
						if ($backup) { copy($dst . '/' . $file, ($dst . '/' . $file . '_' . date("Ymd_His"))); }
					}

					if (function_exists('copy')) {
	                	$result = copy($src . '/' . $file, $dst . '/' . $file);
					} else {
						$result = $this->altcopy($src . '/' . $file, $dst . '/' . $file);
					}
					if (!$result) {
						die("Failed to copy: $src to $dst");
					}
	            }
	        }
	    }
    	closedir($dir);
	}

	function altcopy ($file1, $file2){
		$contentx = file_get_contents($file1);
		$openedfile = fopen($file2, "w");
		fwrite($openedfile, $contentx);
		fclose($openedfile);
		if ($contentx === FALSE) {
			$status = false;
		} else {
			$status = true;
		}
		return $status;
    }

 	private function insertCode ($file, $mark, $code, $position, $match = '', $backup = true) {
		// Check if the mod already exists...
		if (!$match) { $match = $code; }
		if (!$this->searchFileForString($file, $match)) {
			// If not, then add it if the mark exists...
			if ($this->searchFileForString($file, $mark)) {

				// Check if file exists and make a copy for backup with time/date stamp
				if (is_file($file) && file_exists($file)) {
					if ($backup) { copy($file, ($file . '_' . date("Ymd_His"))); }
				} else {
					$this->error['warning'] .= "File not found: $file" . "<br/>";
					return;
				}

				$filedata = file($file);
				foreach ($filedata as $linenum => $line) {
					$line = rtrim($line, "\n") . PHP_EOL;
					$pos = strpos($line, $mark);
					if ($pos !== false) {
						if ($position == 0) { // assume replace if position is 0
							$existing_line = "";
							$filedata[$linenum + $position] = $code;
						} elseif ($position > 0) {
							$existing_line = ($filedata[$linenum + $position]);
							$filedata[$linenum + $position] = ($code . "\n" . $existing_line);
						} else { //$position < 0
							$existing_line = ($filedata[$linenum + $position]);
							$filedata[$linenum + $position] = ($existing_line . $code . "\n");
						}

						$fp = fopen($file, 'w');
						if ($fp) {
							fwrite($fp, implode('', $filedata));
							fclose($fp);
							$this->session->data['success'] .= "File: '$file' modified SUCCESSFULLY" . "<br/>";
							break;
						} else {
							$this->error['warning'] .= "File: '$file' could not be opened for writing" . "<br/>";
						}
					}
				}
				return true;
			}
		} else {
			$this->session->data['success'] .= "File: '$file' already modified" . "<br/>";
		}
	}

	private function searchFileForString($file, $string) {
        // Check if file exists...
		if (!is_file($file) || !file_exists($file)) {
			$this->error['warning'] = "File not found: $file" . "<br/>";
			return;
		}

        // open file to an array
        $fileLines = file($file);

        // loop through lines and look for search term
        $lineNumber = 1;
        foreach($fileLines as $line) {
            $searchCount = substr_count($line, $string);
            if($searchCount > 0) {
                return $lineNumber;
            }
            $lineNumber++;
        }
    }



}
?>